@@ -188,7 +188,7 @@ def _port_mapping_format(port_mappings: List[Job2PortMapping]) -> str:
 
 CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 
-GPULAB_VERSION = '3.5.1'
+GPULAB_VERSION = '3.6.0'
 
 
 def check_and_warn_version(ctx):
@@ -346,7 +346,7 @@ def modify_usage_error():
 @click.version_option(GPULAB_VERSION, message='%(version)s')
 @click.pass_context
 def cli(ctx: click.Context, cert, password, deployment_environment: str, servercert, debug, custom_master_url):
-    """GPULab client version 3.5.1
+    """GPULab client version 3.6.0
 
     \b
     This is the general help. For help with a specific command, try:
@@ -585,7 +585,7 @@ def print_bugreport_context(ctx):
 @click.version_option(GPULAB_VERSION, message='%(version)s')
 @click.pass_context
 def localhost_mode_cli(ctx):
-    """GPULab client version 3.5.1 (in LOCALHOST_MODE!)
+    """GPULab client version 3.6.0 (in LOCALHOST_MODE!)
 
     \b
     Send bugreports, questions and feedback to: gpulab@ilabt.imec.be
@@ -865,6 +865,8 @@ def actual_run_with_job(ctx, project_urn: str, hold: bool, wait_run: bool, wait_
                 email_on_queue=_unique(([client_email_address] if email_queued else []) + job.request.extra.email_on_queue),
                 email_on_run=_unique(([client_email_address] if email_run else []) + job.request.extra.email_on_run),
                 email_on_end=_unique(([client_email_address] if email_done else []) + job.request.extra.email_on_end),
+                email_on_halt=[],
+                email_on_restart=[],
             )
 
     if ctx.obj[VERBOSE_DEBUG]:
@@ -1029,6 +1031,22 @@ def cancel(ctx, job_id):
         raise click.ClickException("Error while cancelling job: {}".format(r.text))
 
 
+@click.command("halt", short_help="Halt a running job. This is used to test the HALTING procedure of a job. "
+                                  "Status change: RUNNING -> MUSTHALT -> HALTING -> FAILURE or HALTED")
+@click.argument('job_id', required=True, type=click.STRING)
+@click.pass_context
+@require_ctx_loggedin_api_client
+def halt_job(ctx, job_id):
+    client = ctx.obj[API_CLIENT]
+    s = client.get_session()
+    r = s.put("{}jobs/{}/state/status".format(client.get_api_url(3), job_id), data='MUSTHALT', timeout=GPULAB_CLIENT_TIMEOUT)
+
+    if r.ok:
+        print(job_id)
+    else:
+        raise click.ClickException("Error while halting job: {}".format(r.text))
+
+
 # variations seen for this commands:  "revoke" "abort" "cancel" "del" "rm"  "edit"
 @click.command("hold", short_help="Hold queued job(s). Status will change from QUEUED to ONHOLD")
 @click.argument('job_id', required=False, type=click.STRING)
@@ -1162,7 +1180,7 @@ def get_debug(ctx, job_id, raw):
                 event = JobEvent.from_dict(e)
 
                 if event.type == JobEventType.STATUS_CHANGE:
-                    print('{} STATE CHANGED -> {}'.format(_date_to_str(event.time), event.new_state.name))
+                    print('{} STATE CHANGED -> {}'.format(_date_to_str(event.time), event.new_status.name if event.new_status else 'unknown new_state'))
                 else:
                     print('{} LOG {}: {}'.format(_date_to_str(event.time), event.type.name, event.msg))
 
@@ -1387,6 +1405,17 @@ def get_info(ctx, job_id: str, raw: bool, user: bool, definition: bool,
                               +_show_rel(job.state.event_times.STARTING, job.state.event_times.ASSIGNED, 'ASSIGNED'))
                         print("{:>20}: {}".format("Running", handle_none(_date_to_str(job.state.event_times.RUNNING)))
                               +_show_rel(job.state.event_times.RUNNING, job.state.event_times.STARTING, 'STARTING'))
+                        if job.state.event_times.MUSTHALT and job.state.event_times.HALTING:
+                            print("{:>20}: {}".format("HALT request", handle_none(_date_to_str(job.state.event_times.MUSTHALT)))
+                                  +_show_rel(job.state.event_times.MUSTHALT, job.state.event_times.RUNNING, 'RUNNING'))
+                            print("{:>20}: {}".format("HALT procedure start", handle_none(_date_to_str(job.state.event_times.HALTING)))
+                                  +_show_rel(job.state.event_times.HALTING, job.state.event_times.MUSTHALT, 'HALT request'))
+                            if job.state.event_times.HALTED:
+                                print("{:>20}: {}".format("HALT success", handle_none(_date_to_str(job.state.event_times.HALTED)))
+                                      +_show_rel(job.state.event_times.HALTED, job.state.event_times.HALTING, 'HALT procedure start'))
+                            if job.state.event_times.FAILED:
+                                print("{:>20}: {}".format("HALT failure", handle_none(_date_to_str(job.state.event_times.FAILED)))
+                                      +_show_rel(job.state.event_times.FAILED, job.state.event_times.HALTING, 'HALT procedure start'))
                         print("{:>20}: {}".format("Ended", handle_none(_date_to_str(job.state.event_times.end_date())))
                               #+_show_rel(job.state.event_times.end_date(), job.state.event_times.RUNNING, 'RUNNING')
                               +_show_ago(job.state.event_times.end_date()))
@@ -1549,7 +1578,9 @@ def actual_wait(ctx, job_id, wait_run, wait_done):
                     print_requires_newline_first = True
 
                 running = job.state.status == JobStatus.STARTING \
-                          or job.state.status == JobStatus.RUNNING
+                          or job.state.status == JobStatus.RUNNING \
+                          or job.state.status == JobStatus.MUSTHALT \
+                          or job.state.status == JobStatus.HALTING
                 prev_status = job.state.status
             else:
                 if print_requires_newline_first:
@@ -1870,6 +1901,7 @@ cli.add_command(run)
 # cli.add_command(queue)
 cli.add_command(rm)
 cli.add_command(cancel)
+cli.add_command(halt_job)
 cli.add_command(hold_job)
 cli.add_command(release)
 cli.add_command(get_log)
@@ -1887,6 +1919,7 @@ localhost_mode_cli.add_command(run_fixed_project)
 # localhost_mode_cli.add_command(queue)
 localhost_mode_cli.add_command(rm)
 localhost_mode_cli.add_command(cancel)
+localhost_mode_cli.add_command(halt_job)
 localhost_mode_cli.add_command(hold_job)
 localhost_mode_cli.add_command(release)
 localhost_mode_cli.add_command(get_log)
